%%

% Función principal. Define los parametros del sistema y llama a las demás funciones
% para resolverlo y graficarlo.

clear variables
close all

clc


% Parametros del sistema.
n_masas  = 50;
m        = 0.1;           % masas [kg]
g        = 10;            % aceleracion de la gravedad [m/s^2]
L        = .1;            % longitud del pendulo [m]
k        = 5;             % constante elastica [N/m]
d        = 0.1;           % separación entre pendulos [m]
extremos = [ 1 1 ];       % ambos extremos fijos (1: fijo; 0: libre) ACLARACION: la condición 'fijo' se aplica a una masa ficticia adyacente a la del borde

% Parámetros del forzante. Se puede elegir sobre qué masas se aplicará, pero la
% dependencia temporal y la amplitud de la fuerza será igual para todas las masas
% seleccionadas.
forzante.frec  = 20; % sqrt(g/L + 4*k/m);      % frecuencia angular del forzante [Hz]
forzante.F0    = 1;                             % amplitud de la fuerza [N]
forzante.masas = [ 1 ] ;                        % vector en el que se indican sobre qué masas se aplica la fuerza. 
forzante.fh    = @(t) cos(forzante.frec*t);     % handle del forzante

% Calcula los puntos de tiempo en los que veremos la solución. Está configurado para
% que podamos ver con buen detalle 30 ciclos de oscilación del forzante.
n_ciclos  = 30;
time_span = linspace(0, n_ciclos*2*pi/forzante.frec, n_ciclos*75);

% % % Acá empieza la movida % % %

% Define el sistema a partir de los parametros y lo guarda en la variable "s".
s = definir_sistema(n_masas, m, g, L, k, d, extremos, forzante);
% Resuelve el sistema homogéneo y luego calcula la solución particular para los
% instantes de tiempo indicados por "time_span". Agrega los resultados dentro de "s".
s = resolver_sistema(s, time_span);

% Luego muestra los resultados
if n_masas < 100
    % dibujar_sistema(s);                 % Dibuja el sistema
    graficar_modos_normales(s);         % Muestra la frecuencia y la forma espacial de cada modo normal
end
graficar_relaciones_de_dispersion(s);   % Muestra las relaciones de dispersión
graficar_forzante(s);                   % Muestra el vector del forzante sobre masas y sobre modos
graficar_evolucion(s);                  % Muestra una animación de la solución particular
