%%

function s = resolver_sistema(s, time_span)

    % Resuelvo la parte homogénea
    
    % -- primero escribo la matriz
    s.matriz_M = calcular_matriz_M(s);
    % -- luego uso una función que me da los autovalores (frecuencias al cuadrado) y autovectores
    % [ s.matriz_D s.matriz_W_cuadrado ] = eig(s.matriz_M, 'nobalance');
    [ s.matriz_D s.matriz_W_cuadrado ] = eig(s.matriz_M);
    % -- obtengo la matriz inversa mediante transposición
    s.matriz_D_inversa  = s.matriz_D';      % <- si son reales es la transpuesta. si son complejos es transpuesta conjugada
    
    % -- obtengo las frecuencias utilizando raiz cuadrada. chequeo que no me queden
    % frecuencias complejas 
    sel                 =  s.matriz_W_cuadrado < 0;
    s.vector_W_cuadrado = -s.matriz_W_cuadrado(sel);
    s.vector_W          = sqrt(s.vector_W_cuadrado);
    
    % % % Con esto ya resolví la parte homogénea. Conozco los modos normales y sus
    % % % frecuencias
    
    % Resuelvo la parte inhomogénea
    
    % -- intervalo de tiempo en que resolveré la evolución
    s.time_span = time_span;
    
    % -- calculo el forzante sobre modos utilizando la matriz D inversa
    s.forzante_F0_sobre_modos                 = s.matriz_D_inversa * (s.forzante_F0_sobre_masas' / s.m);
    % -- obtengo la solución para cada modo aplicando oscilador armónico 1d forzado sin disipación
    s.solucion_amplitudes_forzado_sobre_modos = resolver_oscilador_armonico(s.m, s.vector_W, s.forzante_F0_sobre_modos, s.forzante_frec);
    % -- obtengo la parte temporal del forzante; simplemente es calcular cos(omega*t) para los instantes de tiempo dados por "time_span".
    % la guardo en una variable auxiliar ya que la utilizaré para modos y para masas
    s.evolucion_temporal_forzante             = s.forzante_fh(s.time_span);
    % -- la evolución de los modos se obtiene combinando la solución sobre cada modo con el coseno de omega*t
    s.solucion_forzado_sobre_modos            = s.solucion_amplitudes_forzado_sobre_modos * s.evolucion_temporal_forzante;
    % -- luego obtengo la evolución para cada masa. antitransformo las amplitudes sobre modos obtenidas mediante la solución
    % del oscilador 1d a las coordenadas de las masas (mediante la matriz D). también podria antitransformar toda la
    % evolución temporal pero eso tiene mucho error numérico
    s.solucion_forzado_sobre_masas            = (s.matriz_D * s.solucion_amplitudes_forzado_sobre_modos) * s.evolucion_temporal_forzante;
    % s.solucion_forzado_sobre_masas            = s.matriz_D * s.solucion_forzado_sobre_modos;      % <- tiene mucho error numerico

    return
end

function matriz_M = calcular_matriz_M(s)

    matriz_M = zeros(s.n_masas);
    tam_M    = [ s.n_masas s.n_masas ];

    % indices para las diagonales:
    i_diagonal_central  = 1:s.n_masas;
    j_diagonal_central  = 1:s.n_masas;
    
    i_diagonal_superior = 1:s.n_masas-1;
    j_diagonal_superior = 2:s.n_masas;

    i_diagonal_inferior = j_diagonal_superior;
    j_diagonal_inferior = i_diagonal_superior;
    
    % lleno la diagonal central
    w1_cuadrado = s.k/s.m;
    w0_cuadrado = 2*s.k/s.m + s.g/s.L;
    
    matriz_M( sub2ind(tam_M, i_diagonal_central,  j_diagonal_central ) ) = -w0_cuadrado;
    matriz_M( sub2ind(tam_M, i_diagonal_superior, j_diagonal_superior) ) = w1_cuadrado;
    matriz_M( sub2ind(tam_M, i_diagonal_inferior, j_diagonal_inferior) ) = w1_cuadrado;
    
    % Según cómo sean los extremos, el primer y último elemento diagonal son
    % distintos que los demás
    if s.extremos(1) == 0                           % primer extremo
        matriz_M(1,1) = -(s.k/s.m + s.g/s.L);
    end
    
    if s.extremos(2) == 0                           % último extremo
        matriz_M(end,end) = -(s.k/s.m + s.g/s.L);
    end

    return
end


% Se puede modificar la diagonal en forma progresiva - simulación de la coclea
% Modificando g/L a medida que aumenta la masa, para simular la coclea...')
% matriz_M( sub2ind(tam_M, i_diagonal_central,  j_diagonal_central ) ) = -w0_cuadrado *(1+(1:s.n_masas)*10/s.n_masas);
    